/*
CamineetDemo
    This demo is used to test new concepts for Tiamat framework classes. 
    The objectives are:
        -Extend Area class.
        -Test ideas about game scripting.
        -Test event handling ideas.
        -General UI experimentation.
BaseGameState.java - base game state
    
 Copyright  (C) 2005 Hugues Johnson
 Phantasy Star Copyright  (C) 1988 Sega
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
the GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

package com.huguesjohnson.tiamat.demos.Camineet;

import com.huguesjohnson.serialization.Formatter;
import com.huguesjohnson.serialization.Storable;
import com.huguesjohnson.tiamat.Area;
import java.util.ArrayList;

/** BaseGameState - base game state
 * @author Hugues Johnson
 */
public class BaseGameState implements Storable{

/* begin public properties */
    //the Area the game is currently in
    private Area currentArea;
    /** Returns the Area the game is currently in.
     * @return The Area the game is currently in.
     */
    public Area getCurrentArea(){
        return(this.currentArea);
    }
    /** Sets the Area the game is currently in.
     * @param currentArea The new Area the game is currently in.
     */
    public void setCurrentArea(Area currentArea){
        this.currentArea=currentArea;
    }
    //conditions is a list of things that have happened in the game
    ArrayList<String> conditions;
    /** Add a condition (something that has happened in the game).
     * @param condition The condition to add.
     */
    public void addCondition(String condition){
        this.conditions.add(condition);
    }
    /** Returns all conditions (things that have happened in the game).
     * @return A string array of all conditions (things that have happened in the game).
     */
    public String[] getConditions(){
        return(this.conditions.toArray(new String[0]));
    }
    /** Tests if a specific condition has occured.
     * @param condition The condition to search for.
     * @return True if the specified condition has occured.
     */
    public boolean hasCondition(String condition){
        boolean hascondition=false;
        String[] allConditions=this.getConditions();
        int length=allConditions.length;
        int count=0;
        while((count<length)&&(!hascondition)){
            if(condition.equals(allConditions[count])){
                hascondition=true;
            } else{
                count++;
            }
        }
        return(hascondition);
    }
/* end public properties */
    
/* begin constructors */
    /** Create an empty BaseGameState.
     */
    public BaseGameState(){ 
        this.currentArea=new CDArea();
        this.conditions=new ArrayList<String>();
    }
    /** Create a BaseGameState with conditions but no Area.
     * @param conditions The initial conditions for this BaseGameState.
     */
    public BaseGameState(String[] conditions){ 
        this.currentArea=new CDArea();
        this.conditions=new ArrayList<String>();
        int length=conditions.length;
        for(int index=0;index<length;index++){
            this.addCondition(conditions[index]);
        }
    }
    /** Create a BaseGameState with an Area but no conditions.
     * @param currentArea The new Area the game is currently in.
     */
    public BaseGameState(Area currentArea){ 
        this.currentArea=currentArea;
        this.conditions=new ArrayList<String>();
    }
    /** Create a BaseGameState with conditions and an Area.
     * @param currentArea The new Area the game is currently in.
     * @param conditions The initial conditions for this BaseGameState.
     */
    public BaseGameState(Area currentArea,String[] conditions){ 
        this.currentArea=currentArea;
        this.conditions=new ArrayList<String>();
        int length=conditions.length;
        for(int index=0;index<length;index++){
            this.addCondition(conditions[index]);
        }
    }
/* end constructors */
    
/* begin public methods */
    /** The onDeserialize event should be fired after an object is de-serialized.
     * This allows objects to load transient members, set their working directory, and load Storable members if needed.
     * @param formatter the formatter to use to load any storable members
     * @param absoluteDir the working directory since all paths stored in the object should be relative
     */
    public void onDeserialize(Formatter formatter,String absoluteDir){
        //call onDeserialize for currentArea
        this.currentArea.onDeserialize(formatter,absoluteDir);
    }
/* end public methods */ 
}