/*
CamineetDemo
    This demo is used to test new concepts for Tiamat framework classes. 
    The objectives are:
        -Extend Area class.
        -Test ideas about game scripting.
        -Test event handling ideas.
        -General UI experimentation.
CDArea.java - extension of the Area class with custom properties for this demo
    
 Copyright  (C) 2005 Hugues Johnson
 Phantasy Star Copyright  (C) 1988 Sega
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
the GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

package com.huguesjohnson.tiamat.demos.Camineet;

import com.huguesjohnson.ImageLoader;
import com.huguesjohnson.PathResolver;
import com.huguesjohnson.serialization.Formatter;
import com.huguesjohnson.tiamat.Area;
import com.huguesjohnson.tiamat.demos.Camineet.AreaExit;
import com.huguesjohnson.tiamat.demos.Camineet.ItemListEntry;
import java.awt.image.BufferedImage;

/** CDArea - extension of the Area class with custom properties for this demo
 * @author Hugues Johnson
 */
public class CDArea extends Area{
/* begin public properties */
    //the exits from this Area
    private AreaExit[] exits;
    /** Retuns all the exits from this Area.
     * @return All the exits from this Area.
     */
    public AreaExit[] getExits(){
        return(this.exits);
    }
    /** Sets all the exits from this Area.
     * @param exits All the exits from this Area.
     */
    public void setExits(AreaExit[] exits){
        this.exits=exits;
    }
    //list of all the Items in this Area
    private ItemListEntry[] itemList;
    /** Returns a list of all the Items in the Area.
     * @return A list of al the Items in the Area.
     */
    public ItemListEntry[] getItemList(){
        return(this.itemList);
    }
    /** Sets a list of all the Items in the Area.
     * @param itemList The list of al the Items in the Area.
     */
    public void setItemList(ItemListEntry[] itemList){
        this.itemList=itemList;
    }
    //the relative path to the background music for this Area
    private String bgMusicPath;
    /** Returns the relative path to the background music.
     * @return The relative path to the background music.
     */
    public String getBgMusicPath(){
        return(this.bgMusicPath);
    }
    /** Sets the relative path to the background music.
     * @param bgMusicPath The relative path to the background music.
     */
    public void setBgMusicPath(String bgMusicPath){
        this.bgMusicPath=new String(bgMusicPath);
    }
    /** Returns the absolute path to the background music.
     * @return The absolute path to the background music.
     */
    public String getBgMusicPathAbsolute(){
        return(PathResolver.getAbsolutePath(this.absoluteDir,this.bgMusicPath));
    }
    //relative path to the background image for this CDArea, setting this reloads the actual image
    private String bgImagePath;
    /** Sets the relative path to the background image.
     * @param bgImagePath The relative path to the background image.
     */
    public void setBgImagePath(String bgImagePath){ 
        this.bgImagePath=bgImagePath; 
        this.bgImage=ImageLoader.loadBufferedImage(this.getBgImagePathAbsolute());
    }
    /** Returns the relative path to the background image.
     * @return The relative path to the background image.
     */
    public String getBgImagePath(){ 
        return(this.bgImagePath); 
    }
    /** Returns the absolute path to the background image.
     * @return The absolute path to the background image.
     */
    public String getBgImagePathAbsolute(){ 
        return(PathResolver.getAbsolutePath(this.absoluteDir,this.bgImagePath)); 
    }
    //background image for this CDArea, change by setting bgImagePath
    private transient BufferedImage bgImage;
    /** Returns the background image for this CDArea.
     * @return The background image for this CDArea.
     */
    public BufferedImage getBgImage(){ 
        return(this.bgImage);
    }
    //overlay image for this CDArea, change by setting overlayImagePath
    private transient BufferedImage overlayImage;
    /** Returns the overlay image for this CDArea.
     * @return The overlay image for this CDArea.
     */
    public BufferedImage getOverlayImage(){ 
        return(this.overlayImage);
    }
    //relative path to the overlay image for this CDArea, setting this reloads the actual image
    private String overlayImagePath;
    /** Sets the relative path to the overlay image.
     * @param bgImagePath The relative path to the overlay image.
     */
    public void setOverlayImagePath(String overlayImagePath){ 
        this.overlayImagePath=overlayImagePath; 
        this.overlayImage=ImageLoader.loadBufferedImage(this.getOverlayImagePathAbsolute());
    }
    /** Returns the relative path to the overlay image.
     * @return The relative path to the overlay image.
     */
    public String getOverlayImagePath(){ 
        return(this.overlayImagePath); 
    }
    /** Returns the absolute path to the overlay image.
     * @return The absolute path to the overlay image.
     */
    public String getOverlayImagePathAbsolute(){
        return(PathResolver.getAbsolutePath(this.absoluteDir,this.overlayImagePath)); 
    }
/* end public properties */

/* begin overides */
    /** The onDeserialize event should be fired after an object is de-serialized.
     * This allows objects to load transient members, set their working directory, and load Storable members if needed.
     * @param formatter the formatter to use to load any storable members
     * @param absoluteDir the working directory since all paths stored in the object should be relative
     */
    public void onDeserialize(Formatter formatter,String absoluteDir){
        super.onDeserialize(formatter,absoluteDir);
        //call onDeserialize for all Items in this Area
        if(this.itemList==null){
            this.itemList=new ItemListEntry[0];
        } else{
            int count=this.itemList.length;
            for(int index=0;index<count;index++){
                this.itemList[index].getItem().onDeserialize(formatter,absoluteDir);
            }
        } 
        //load the images
        this.bgImage=ImageLoader.loadBufferedImage(this.getBgImagePathAbsolute());
        this.overlayImage=ImageLoader.loadBufferedImage(this.getOverlayImagePathAbsolute());
    }
/* end overides */
}