/*
CamineetDemo
    This demo is used to test new concepts for Tiamat framework classes. 
    The objectives are:
        -Extend Area class.
        -Test ideas about game scripting.
        -Test event handling ideas.
        -General UI experimentation.
CDGameState.java - extension of BaseGameState with attributes specific to this demo
    
 Copyright  (C) 2005 Hugues Johnson
 Phantasy Star Copyright  (C) 1988 Sega
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
the GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

package com.huguesjohnson.tiamat.demos.Camineet;

import com.huguesjohnson.tiamat.Area;

/** CDGameState - extension of BaseGameState with attributes specific to this demo
 * @author  Hugues Johnson
 */
public class CDGameState extends BaseGameState{
/* begin public static constants */
    /** constant for daytime */
    public final static int TIME_DAY=0;
    /** constant for nighttime */
    public final static int TIME_NIGHT=1;
/* end public static constants */

/* begin public properties */
    //what time it is
    private int timeOfDay;
    /** Returns one of the time constants representing what time of day it is.
     * @return One of the time constants representing what time of day it is.
     */
    public int getTimeOfDay(){
        return(this.timeOfDay);
    }
    /** Sets the time of day it is.
     * @param timeOfDay One of the time constants representing what time of day it is.
     */
    public void setTimeOfDay(int timeOfDay){
        if((timeOfDay==CDGameState.TIME_DAY)||(timeOfDay==CDGameState.TIME_NIGHT)){
            this.timeOfDay=timeOfDay;
        }
    }
    //the name of the current area
    private String currentAreaName;
    /** Returns the name of the current area.
     * @return The name of the current area.
     */
    public String getCurrentAreaName(){
        return(this.currentAreaName);
    }
    /** Sets the name of the current area.
     * @param areaName The new name of the current area.
     */
    public void setCurrentAreaName(String areaName){
        this.currentAreaName=new String(areaName);
    }
/* end public properties */

/* begin constructors */
    /** Create an empty CDGameState.
     */
    public CDGameState(){ 
        super();
        this.timeOfDay=CDGameState.TIME_DAY;
    }
    /** Create a CDGameState with conditions but no Area.
     * @param conditions The initial conditions for this BaseGameState.
     */
    public CDGameState(String[] conditions){ 
        super(conditions);
        this.timeOfDay=CDGameState.TIME_DAY;
    }
    /** Create a BaseGameState with an Area but no conditions.
     * @param currentArea The new Area the game is currently in.
     */
    public CDGameState(Area currentArea){ 
        super(currentArea);
        this.timeOfDay=CDGameState.TIME_DAY;
    }
    /** Create a BaseGameState with conditions and an Area.
     * @param currentArea The new Area the game is currently in.
     * @param conditions The initial conditions for this BaseGameState.
     */
    public CDGameState(Area currentArea,String[] conditions){ 
        super(currentArea,conditions);
        this.timeOfDay=CDGameState.TIME_DAY;
    }
    /** Create an empty CDGameState.
     * @param timeOfDay One of the time constants representing what time of day it is.
     */
    public CDGameState(int timeOfDay){ 
        super();
        this.setTimeOfDay(timeOfDay);
    }
    /** Create a CDGameState with conditions but no Area.
     * @param conditions The initial conditions for this BaseGameState.
     * @param timeOfDay One of the time constants representing what time of day it is.
     */
    public CDGameState(String[] conditions,int timeOfDay){ 
        super(conditions);
        this.setTimeOfDay(timeOfDay);
    }
    /** Create a BaseGameState with an Area but no conditions.
     * @param currentArea The new Area the game is currently in.
     * @param timeOfDay One of the time constants representing what time of day it is.
     */
    public CDGameState(Area currentArea,int timeOfDay){ 
        super(currentArea);
        this.setTimeOfDay(timeOfDay);
    }
    /** Create a BaseGameState with conditions and an Area.
     * @param currentArea The new Area the game is currently in.
     * @param conditions The initial conditions for this BaseGameState.
     * @param timeOfDay One of the time constants representing what time of day it is.
     */
    public CDGameState(Area currentArea,String[] conditions,int timeOfDay){ 
        super(currentArea,conditions);
        this.setTimeOfDay(timeOfDay);
    }
/* end constructors */
}