/*
CamineetDemo
    This demo is used to test new concepts for Tiamat framework classes. 
    The objectives are:
        -Extend Area class.
        -Test ideas about game scripting.
        -Test event handling ideas.
        -General UI experimentation.
CamineetDemoFrame.java - main frame for the Camineet demo
    
 Copyright  (C) 2005 Hugues Johnson
 Phantasy Star Copyright  (C) 1988 Sega
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
the GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

package com.huguesjohnson.tiamat.demos.Camineet;

import com.huguesjohnson.tiamat.Event;
import com.huguesjohnson.tiamat.Selection;
import com.huguesjohnson.tiamat.graphics.Theme;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.image.BufferedImage;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JViewport;
import javax.swing.ScrollPaneConstants;
import javax.swing.border.LineBorder;

/** CamineetDemoFrame - main frame for the Camineet demo
 * Protected code segments were generated by the NetBeans 3.6 form editor.
 * @author  Hugues Johnson
 */
public class CamineetDemoFrame extends javax.swing.JFrame{
	/** Determines if a de-serialized file is compatible with this class. See <a href="http://java.sun.com/j2se/1.5.0/docs/guide/serialization/spec/class.html">Java Object Serialization Specifications</a>. */
	private static final long serialVersionUID=6605059320043201673L;
	
	//images that are being displayed
    private BufferedImage bgImage;
    private BufferedImage characterImage;
    private BufferedImage overlayImage;
    
    //the theme
    private Theme currentTheme;
    
    //the font
    private Font currentFont;
    
    //the selections
    private Selection[] selections;
    private JRadioButton[] menu; 
    
    //maximum number of menu items
    public final static int MAX_MENU=10;
    
    //how much text should scroll by
    public final static int VSCROLL=100;
    
    /** Creates new form CamineetDemoFrame */
    public CamineetDemoFrame(Theme theme,Font font){
        initComponents();
        this.applyTheme(this,theme,font);
        this.centerWindow();
        this.initMenu();
//uncomment the next set of lines to test scrolling        
//        String longText="The second time around, it wasn't even finished yet. They were still under construction." +
//        "\nA construction job of that magnitude would require a helluva lot more manpower than the Imperial army had to offer." +
//        "\nI'll bet there were independent contractors working on that thing: plumbers, aluminum siders, roofers." +
//        "\nIn order to get it built quickly and quietly they'd hire anybody who could do the job." +
//        "\nDo you think the average storm trooper knows how to install a toilet main?" +
//        "\nAll they know is killing and white uniforms." +
//        "\nAll those innocent contractors hired to do a job were killed - casualties of a war they had nothing to do with." +
//        "\nAll right, look-you're a roofer, and some juicy government contract comes your way; you got the wife and kids and the two-story in suburbia." +
//        "\nThis is a government contract, which means all sorts of benefits." +
//        "\nAll of a sudden these left-wing militants blast you with lasers and wipe out everyone within a three-mile radius." +
//        "\nYou didn't ask for that. " +
//        "\nYou have no personal politics. " +
//        "\nYou're just trying to scrape out a living.";
//        this.drawText(longText);
    }
    
     /** centers the window in the middle of the screen
     */     
     private void centerWindow(){
          Dimension screenDimension=Toolkit.getDefaultToolkit().getScreenSize();
          Rectangle windowDimension=this.getBounds();
          this.setLocation((screenDimension.width-windowDimension.width)/2,(screenDimension.height-windowDimension.height)/2);
     }
    
     /* apply the theme */
     private void applyTheme(Component component,Theme theme,Font font){
        this.currentTheme=theme;
        this.currentFont=font;
        component.setBackground(theme.getColorScheme().getBackColor());
        component.setForeground(theme.getColorScheme().getForeColor());
        component.setFont(font);
        //if component is a container, call applyTheme for its children
        if(component instanceof Container){
            Container container=(Container)component;
            int count=container.getComponentCount();
            if(count>0){
                Component[] children=container.getComponents();
                for(int index=0;index<count;index++){
                    this.applyTheme(children[index],theme,font);
                }
            }
            //set the border colors, not the best way to do this but works for this demo
            if(container instanceof JPanel){
                JPanel panel=(JPanel)container;
                //replace the border for anything with a border
                if(panel.getBorder()!=null){
                    panel.setBorder(new LineBorder(theme.getColorScheme().getBorderColor(),4,true));
                }
            } else if(container instanceof JScrollPane){
                JScrollPane scrollPane=(JScrollPane)container;
                //replace the border for anything with a border
                if(scrollPane.getBorder()!=null){
                    scrollPane.setBorder(new LineBorder(theme.getColorScheme().getBorderColor(),4,true));
                }
            }
        }
        //if component is a JRadioButton, set the button images
        if(component instanceof JRadioButton){
            JRadioButton radioButton=(JRadioButton)component;
            BufferedImage image=theme.getRadioButtonImage();
            if(image!=null){
                radioButton.setIcon(new ImageIcon(image));
            }
            image=theme.getRadioButtonSelectedImage();
            if(image!=null){            
                radioButton.setSelectedIcon(new ImageIcon(image));
            }
        } else if(component instanceof JTextArea){
            JTextArea textArea=(JTextArea)component;
            textArea.setDisabledTextColor(theme.getColorScheme().getForeColor());
        }
     }
     
     /* clear the menu */
     private void clearSelections(){
        int length=this.menu.length;
        for(int index=0;index<length;index++){
            this.menu[index].setVisible(false);
        }
     }
     
     /** Set the selections (menu). 
      * @param selections The new selections to display.
      */
     public void setSelections(Selection[] selections){
         this.clearSelections();
         if(selections!=null){
            int length=selections.length;
            if(length>0){
                this.selections=new Selection[length];
                System.arraycopy(selections,0,this.selections,0,length);
                for(int index=0;index<length;index++){
//not working as expected, attempt to scroll text on menu, font does not get applied though..                
//                    StringBuffer text=new StringBuffer("<html>");
////                    text.append("<font face=\""+this.currentFont.getFontName()+"\"");
//                    text.append(">");
////                    size="6">
//                    text.append(selections[index].getCaption());
//                    text.append("</font>");
//                    text.append("</html>");
//                    this.menu[index].setText(text.toString());
                    this.menu[index].setText(selections[index].getCaption());
                    this.menu[index].setVisible(true);
                }
                this.menu[0].setSelected(true);
                this.menu[0].requestFocus();
            }
         }
    }

     /* initialize the menu */
     public void initMenu(){
        int length=MAX_MENU;
        this.selections=new Selection[length];
        this.menu=new JRadioButton[length];
        double maxHeight=(this.panelMenu.getMaximumSize().getHeight()/length)*.9;
        double maxWidth=this.panelMenu.getMaximumSize().getWidth()*.9;
        for(int index=0;index<length;index++){
            this.menu[index]=new JRadioButton("");
            this.menu[index].setMaximumSize(new Dimension((int)maxWidth,(int)maxHeight));
            this.menu[index].setMinimumSize(new Dimension((int)maxWidth,(int)maxHeight));
            this.menu[index].setPreferredSize(new Dimension((int)maxWidth,(int)maxHeight));
            this.menu[index].setBorderPainted(false);
            this.menu[index].setFocusPainted(false);
            this.menu[index].addKeyListener(new KeyListener(){
                public void keyPressed(KeyEvent keyEvent){ handleKeyEvent(keyEvent.getKeyCode()); }
                public void keyReleased(KeyEvent keyEvent){ }
                public void keyTyped(KeyEvent keyEvent){ }
            });
            this.menu[index].setBackground(this.currentTheme.getColorScheme().getBackColor());
            this.menu[index].setForeground(this.currentTheme.getColorScheme().getForeColor());
            this.menu[index].setFont(this.currentFont);
            BufferedImage image=this.currentTheme.getRadioButtonImage();
            if(image!=null){
                this.menu[index].setIcon(new ImageIcon(image));
            }
            image=this.currentTheme.getRadioButtonSelectedImage();
            if(image!=null){            
                this.menu[index].setSelectedIcon(new ImageIcon(image));
            }
            this.buttonGroupSelections.add(this.menu[index]);
            this.panelMenu.add(this.menu[index]);            
        }
        this.menu[0].setSelected(true);
    }

     public void paint(java.awt.Graphics g){
        super.paint(g);
        this.redrawArea();
     }
     
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        this.buttonGroupSelections = new javax.swing.ButtonGroup();
        this.panelTop = new javax.swing.JPanel();
        this.panelArea = new javax.swing.JPanel();
        this.panelVSpacer = new javax.swing.JPanel();
        this.panelMenu = new javax.swing.JPanel();
        this.panelHSpacer = new javax.swing.JPanel();
        this.panelBottom = new javax.swing.JPanel();
        this.scrollPaneText = new javax.swing.JScrollPane();
        this.textArea = new javax.swing.JTextArea();

        this.getContentPane().setLayout(new javax.swing.BoxLayout(getContentPane(), javax.swing.BoxLayout.Y_AXIS));

        this.setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        this.setTitle("Camineet Demo");
        this.setIconImage((new javax.swing.ImageIcon(getClass().getResource("/com/huguesjohnson/tiamat/demos/Camineet/images/ui/icon.gif"))).getImage());
        this.setName("frame");
        this.setResizable(false);
        this.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                formKeyTyped(evt);
            }
        });
        this.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });
        this.addWindowFocusListener(new java.awt.event.WindowFocusListener() {
            public void windowGainedFocus(java.awt.event.WindowEvent evt) {
                formWindowGainedFocus(evt);
            }
            public void windowLostFocus(java.awt.event.WindowEvent evt) {
            }
        });

        this.panelTop.setLayout(new javax.swing.BoxLayout(this.panelTop, javax.swing.BoxLayout.X_AXIS));

        this.panelArea.setMaximumSize(new java.awt.Dimension(260, 196));
        this.panelArea.setMinimumSize(new java.awt.Dimension(260, 196));
        this.panelArea.setPreferredSize(new java.awt.Dimension(260, 196));
        this.panelTop.add(this.panelArea);

        this.panelVSpacer.setMaximumSize(new java.awt.Dimension(4, 4));
        this.panelVSpacer.setMinimumSize(new java.awt.Dimension(4, 4));
        this.panelVSpacer.setPreferredSize(new java.awt.Dimension(4, 4));
        this.panelTop.add(this.panelVSpacer);

        this.panelMenu.setLayout(new javax.swing.BoxLayout(this.panelMenu, javax.swing.BoxLayout.Y_AXIS));

        this.panelMenu.setBorder(new javax.swing.border.LineBorder(new java.awt.Color(0, 0, 0), 4, true));
        this.panelMenu.setMaximumSize(new java.awt.Dimension(160, 196));
        this.panelMenu.setMinimumSize(new java.awt.Dimension(160, 196));
        this.panelMenu.setPreferredSize(new java.awt.Dimension(160, 196));
        this.panelTop.add(this.panelMenu);

        this.getContentPane().add(this.panelTop);

        this.panelHSpacer.setMaximumSize(new java.awt.Dimension(4, 4));
        this.panelHSpacer.setMinimumSize(new java.awt.Dimension(4, 4));
        this.panelHSpacer.setPreferredSize(new java.awt.Dimension(4, 4));
        this.getContentPane().add(this.panelHSpacer);

        this.panelBottom.setLayout(new javax.swing.BoxLayout(this.panelBottom, javax.swing.BoxLayout.X_AXIS));

        this.scrollPaneText.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        this.scrollPaneText.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
        this.scrollPaneText.setFocusable(false);
        this.scrollPaneText.setMinimumSize(new java.awt.Dimension(24, 120));
        this.scrollPaneText.setPreferredSize(new java.awt.Dimension(4, 120));
        this.scrollPaneText.setRequestFocusEnabled(false);
        this.scrollPaneText.setWheelScrollingEnabled(false);
        this.scrollPaneText.setEnabled(false);
        this.scrollPaneText.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                scrollPaneTextKeyTyped(evt);
            }
        });

        this.textArea.setEditable(false);
        this.textArea.setLineWrap(true);
        this.textArea.setWrapStyleWord(true);
        this.textArea.setFocusable(false);
        this.textArea.setMaximumSize(new java.awt.Dimension(2147483647, 120));
        this.textArea.setRequestFocusEnabled(false);
        this.textArea.setAutoscrolls(false);
        this.textArea.setEnabled(false);
        this.textArea.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                textAreaKeyTyped(evt);
            }
        });

        this.scrollPaneText.setViewportView(this.textArea);

        this.panelBottom.add(this.scrollPaneText);

        this.getContentPane().add(this.panelBottom);

        this.pack();
    }//GEN-END:initComponents

    public void textAreaKeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_textAreaKeyTyped
        this.handleKeyEvent(evt.getKeyCode());
    }//GEN-LAST:event_textAreaKeyTyped

    public void scrollPaneTextKeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_scrollPaneTextKeyTyped
        this.handleKeyEvent(evt.getKeyCode());
    }//GEN-LAST:event_scrollPaneTextKeyTyped

    public void formWindowGainedFocus(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowGainedFocus
        this.redrawArea();
    }//GEN-LAST:event_formWindowGainedFocus

    public void formKeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_formKeyTyped
        this.handleKeyEvent(evt.getKeyCode());
    }//GEN-LAST:event_formKeyTyped
    
    /** Exit the Application */
    public void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        this.onWindowClose();
    }//GEN-LAST:event_exitForm
    
    /** Event that fires when the frame is closed.
     * Override this if using this frame from an external application. 
     */
    public void onWindowClose(){
        System.exit(0);
    }

    /** Draw an area.. there's probably a better way to this.
     * @param bgImage The background for the area.
     * @param character The character in this area, set to null if no character.
     * @param overlay The overlay image for this area, set to null if none.
     */
    public void drawArea(BufferedImage bgImage,BufferedImage characterImage,BufferedImage overlayImage){
        this.bgImage=bgImage;
        this.characterImage=characterImage;
        this.overlayImage=overlayImage;
        this.redrawArea();
    }
    
    /** Redraw the area being displayed with the current images.. there's probably a better way to this.
     */
    private void redrawArea(){
        Graphics2D g2=(Graphics2D)this.panelArea.getGraphics();
        //draw the background
        if(this.bgImage!=null){
            g2.drawImage(this.bgImage,0,0,this.panelArea);
        }
        //draw the character
        if(this.characterImage!=null){
            //center the character
            int panelWidth=this.panelArea.getWidth();
            int characterWidth=this.characterImage.getWidth();
            int x=(panelWidth-characterWidth)/2;
            //align them to the bottom <- this is the part that's screwy
            int bottom=26;
            int characterHeight=this.characterImage.getHeight();
            int panelHeight=this.panelArea.getHeight();
            int y=panelHeight-(characterHeight+bottom);
            //now draw the character
            g2.drawImage(this.characterImage,x,y,this.panelArea);
        }
        //draw the overlay
        if(this.overlayImage!=null){
            g2.drawImage(this.overlayImage,0,0,this.panelArea);
        }
    }
    
    /** Draw text in the lower text pane.
     * @param text The text to draw in the lower text pane.
     */
    public void drawText(String text){
        //draw the text
        this.textArea.setText(text);
        //scroll to the top
        this.textArea.setCaretPosition(0);
        this.scrollPaneText.getViewport().setViewPosition(new Point(0,0));
    }
    
    /** Handle key events.
     * @param keyCode The code for the key that was pressed.
     */
    public void handleKeyEvent(int keyEvent){
        switch(keyEvent){
            case KeyEvent.VK_PAGE_UP:{
                //scroll text up
                JViewport viewport=this.scrollPaneText.getViewport();
                Point currentView=viewport.getViewPosition();
                int cx=(int)currentView.getX();
                int cy=(int)currentView.getY();
                int nx=cx;
                int ny=cy-CamineetDemoFrame.VSCROLL;
                if(ny<0){ 
                    ny=0; 
                }
                if(ny<cy){
                    Point viewPosition=new Point(nx,ny);
                    viewport.setViewPosition(viewPosition);
                }
                break;                
            }
            case KeyEvent.VK_PAGE_DOWN:{
                //scroll text down
                JViewport viewport=this.scrollPaneText.getViewport();
                Point currentView=viewport.getViewPosition();
                int cx=(int)currentView.getX();
                int cy=(int)currentView.getY();
                int nx=cx;
                int ny=cy+CamineetDemoFrame.VSCROLL;
                int textAreaHeight=(int)this.textArea.getSize().getHeight();
                int scrollPaneHeight=(int)this.scrollPaneText.getSize().getHeight();
                int bottom=this.scrollPaneText.getInsets().bottom;
                int top=this.scrollPaneText.getInsets().top;
                scrollPaneHeight-=(top+bottom);
                int maxy=textAreaHeight-scrollPaneHeight;
                if(ny>maxy){ 
                    ny=maxy; 
                }
                if(ny>cy){
                    Point viewPosition=new Point(nx,ny);
                    viewport.setViewPosition(viewPosition);
                }
                break;
            }
            case KeyEvent.VK_UP:{
                int selectedIndex=this.getSelectionIndex();
                if(selectedIndex>0){
                    this.menu[selectedIndex-1].setSelected(true);
                } else { //scroll back to the bottom
                    int last=this.menu.length-1;
                    boolean found=false;
                    while((last>selectedIndex)&&(!found)){
                        if(this.menu[last].isVisible()){
                            this.menu[last].setSelected(true);
                            found=true;
                        }
                        last--;
                    }
                }
                break;
            }
            case KeyEvent.VK_DOWN:{
                int length=this.menu.length-1;
                int selectedIndex=this.getSelectionIndex();
                if((selectedIndex<length)&&(this.menu[selectedIndex+1].isVisible())){
                    this.menu[selectedIndex+1].setSelected(true);
                } else{ //scroll back to the top
                    if(this.menu[0].isVisible()){
                        this.menu[0].setSelected(true);
                    }
                }
                break;
            }
            case KeyEvent.VK_ENTER:{
                int selectedIndex=this.getSelectionIndex();
                if((selectedIndex>=0)&&(selectedIndex<this.selections.length)){
                    this.postEvent(this.selections[selectedIndex].getEvent());
                }
                break;
            }
        }
    }

    /** Returns the index of the selection chosen by the user.
     * @return The index of the selection chosen by the user.
     */
    private int getSelectionIndex(){
        int selectionIndex=-1;
        if(this.menu!=null){
            int length=this.menu.length;
            int index=0;
            while((index<length)&&(selectionIndex==-1)){
                if(this.menu[index].isSelected()){
                    selectionIndex=index;
                } else{
                    index++;
                }
            }
        }
        return(selectionIndex);
    }    
    
    /** Post an event entered by the user.
     * Override this if using this frame from an external application. 
     * @param event the Event entered by the user.
     */
    public void postEvent(Event event){ }
    
    /** just around for testing
     * @param args the command line arguments
     */
    public static void main(String args[]){
        CamineetDemoFrame frame=new CamineetDemoFrame(new Theme(),new Font(null,Font.PLAIN,12));
        frame.setVisible(true);
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroupSelections;
    private javax.swing.JPanel panelArea;
    private javax.swing.JPanel panelBottom;
    private javax.swing.JPanel panelHSpacer;
    private javax.swing.JPanel panelMenu;
    private javax.swing.JPanel panelTop;
    private javax.swing.JPanel panelVSpacer;
    private javax.swing.JScrollPane scrollPaneText;
    private javax.swing.JTextArea textArea;
    // End of variables declaration//GEN-END:variables
}